from argparse import ArgumentParser
import os
from dvc_cc.bcolors import *
import yaml

def parse_one_of(content):
    if len(content["dtype"]) == 0:
        return None
    elif len(content["dtype"]) == 1:
        r = content["dtype"][0]
    else:
        if "question" in content:
            print(content["question"])
        else:
            print("Please choose one of the following options:")
        dtype_dict = {}
        print("!!!!DTYPE:", content["dtype"])
        for o in content["dtype"]:
            if "name" in o and 'help' in o:
                dtype_dict[o['name']] = o
                print(o['name'] + ': ' + o['help'])
        user_input = None
        while user_input is None:
            user_input = input("Your choise: ").lower()
            if  user_input not in dtype_dict:
                print('Error, please use one of the above terms.')
                user_input = None
        r = dtype_dict[user_input]
    return parse(r)

def parse(content, interactive=True):
    """
    This helper becomes a prototype of for a config file and fill it.

    :param content: The content of the prototype config file.
    :param interactive: If True than the user will get interactive ask what he wants, if False it will take always the
                        default values or None as values.
    :return: You get the filled settings.
    """
    result = {}
    if type(content) is dict:
        if "dtype" in content:
            if type(content["dtype"]) is list:
                parse_one_of(content)

            """
                #print("DTYPE=LIST")
            elif content["dtype"].lower() is "string":
                print("DTYPE=STRING")
            elif content["dtype"].lower() is "int":
                print("DTYPE=int")
            elif content["dtype"].lower() is "int>0":
                print("DTYPE=int>0")
            elif content["dtype"].lower() is "int>=0":
                print("DTYPE=int>=0")
            """
        else:
            summary = {}
            for k in content.keys():
                if type(k) is dict:
                    summary[k] = parse(content[k])


            if "value" in content:
                result[]


    # if dtype is a list
    # if dtype is int, int>0, ...


from argparse import ArgumentParser
import matplotlib.pyplot as plt
import numpy as np
from dvc_cc.bcolors import *


DESCRIPTION = 'This script represents beta distributions. You can use this to work with '+bcolors.OKBLUE+'dvc-cc run'+bcolors.ENDC+ \
              ' and see from which distribution the hyperparemeter is drawn.'

def main():
    parser = ArgumentParser(description=DESCRIPTION)
    args = parser.parse_args()

    print('Here matplotlib.pyplot.show() will be called.')
    print('Use CTRL+C to stop this script.')
    min = float(input('Min-Value: '))
    max = float(input('Max-Value: '))
    results = {}
    while True:
        a = float(input('A-Value: '))
        b = float(input('B-Value: '))
        print('Plotting')
        values = np.random.beta(a, b, size=1000000)
        values *= max - min
        values += min
        r = np.histogram(values,bins=100, density=True)
        r = r[0] ,  [(r[1][i]+r[1][i+1])/2.0 for i in range(len(r[1])-1)]
        name = 'A:'+str(a)+' ; B:'+str(b)
        results[name] = r
        for key in results:
            r = results[key]
            plt.bar(r[1],r[0], width=r[1][1]-r[1][0], alpha=0.4, label=key)
        plt.legend()
        plt.show()





#!/usr/bin/env python3

import json

def read_parameters_from_parametercell(path_to_ipynb, cell_tag_name='parameters'):
    """
     search for cells with the tag "parameters" and read this cells.
     returns a list of parameters back with three values each parameter: varname, vartype, help message
     if no parameter was found, it will return an empty list.
     """
    with open(path_to_ipynb) as f:
        notebook = json.load(f)
    founded_parameters = []
    cells = notebook["cells"]
    for i in range(len(cells)):
        c = cells[i]
        if 'tags' in c['metadata'] and cell_tag_name in c['metadata']['tags']:
            commentar = ''
            for line_original in c['source']:
                line = line_original.strip()
                if line.find('#') >= 0:
                    commentar = commentar + line[line.find('#')+1:].strip()
                    line = line[:line.find('#')].strip()

                if line.find('=') > 0:
                    line_sep = line.split('=')
                    varname = line_sep[0].strip()
                    default_value = line_sep[1].strip()

                    if line_original.find('one_of:') > 0:
                        one_of_values = line_original.split('one_of:')[1].replace('"',"'")

                        if one_of_values.find("'") >= 0:
                            # first case, it was used ' at the beginning and ending of each possible value
                            one_of_list = one_of_values.split("'")[1::2]
                            do_strip = False
                        elif one_of_values.find(",") >= 0:
                            # second case, it was used , to seperate the possible values
                            one_of_list = one_of_values.split(",")
                            do_strip = True
                        else:
                            # third case, it was used space to seperate the values
                            one_of_list = one_of_values.split(" ")
                            do_strip = True

                        # build datatyoe:
                        datatype = '['
                        for constant in one_of_list:
                            if do_strip:
                                constant = constant.strip()
                                if len(constant) > 0:
                                    datatype = datatype + constant + ','
                            else:
                                datatype = datatype + constant + ','
                        if len(datatype) == 1:
                            raise ValueError('You need to set values for one_of constant in your jupyter notebook.\n'
                                             'Here are some examples:\n'
                                             'paraneter1 = 3.141 # one_of: 3.14159, 4.6692, 42\n'
                                             'paraneter2 = 1 # one_of: 1, 2, 3\n'
                                             'paraneter3 = 1 # one_of: \'1\', \'2\', \'3\'\n'
                                             'paraneter4 = 1 # one_of: 1 2 3'
                                             'paraneter5 = \'text1\' # one_of: \'text\',\'text2\',\'text3\'\n'
                                             'paraneter6 = \'text1\' # one_of: text, text2, text3\n'
                                             'paraneter7 = \'text1\' # one_of: text text2 text3\n'
                                             'paraneter8 = \'text1\' # SOME DESCRIPTION one_of: text text2 text3\n'
                                             '\nThe one_of values from which the user can choose, is in 2-4 and 5-8 '
                                             'the same.')
                        datatype = datatype[:-1] + ']'

                    elif default_value.find("'") >= 0 or default_value.find('"')  >= 0:
                        datatype = 'not-supported'
                    elif default_value.isdigit():
                        datatype = 'int'
                    elif default_value.startswith('-') and default_value[1:].isdigit():
                        datatype = 'int'
                    elif default_value.find('.') >= 0:
                        datatype = 'float'
                    else:
                        datatype = 'not-supported'

                    if datatype != 'not-supported':
                        founded_parameters.append([varname, datatype, commentar])

                    commentar = ''
    return founded_parameters


def read_definitions_from_parametercell(path_to_ipynb, cell_tag_name='outputs'):
    """
     search for cells with the tag "parameters" and read this cells.
     returns a list of parameters back with three values each parameter: varname, vartype, help message
     if no parameter was found, it will return an empty list.
     """
    with open(path_to_ipynb) as f:
        notebook = json.load(f)
    founded_parameters = []
    cells = notebook["cells"]
    for i in range(len(cells)):
        c = cells[i]
        if 'tags' in c['metadata'] and cell_tag_name in c['metadata']['tags']:
            for line_original in c['source']:
                line = line_original.strip()
                if line.find('#') >= 0:
                    line = line[:line.find('#')].strip()

                if line.find('=') > 0:
                    line_sep = line.split('=')
                    varname = line_sep[0].strip()
                    value = line_sep[1].strip()

                    if (value.startswith("'") or value.startswith("'")) and (value.endswith("'") or value.endswith("'")):
                        founded_parameters.append([varname, value[1:-1]])
    return founded_parameters